﻿namespace U2UConsult.EF40.OptimisticConcurrency.Sample
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Data;
    using System.Data.Objects;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private ObservableCollection<FerengiRule> rules =
            new ObservableCollection<FerengiRule>();

        private List<FerengiRule> deletedRules = new List<FerengiRule>();

        private ConflictResolution conflictResolution = ConflictResolution.Default;

        public MainWindow()
        {
            InitializeComponent();

            this.RefreshRules();
        }

        private enum ConflictResolution
        {
            Default,
            ServerWins,
            ClientWins
        }

        private void RefreshRules()
        {
            this.deletedRules.Clear();

            this.RulesDataGrid.ItemsSource = null;
            this.RulesDataGrid.Items.Clear();
            this.rules.Clear();

            using (DockOfTheBayEntities entities = new DockOfTheBayEntities())
            {
                foreach (var rule in entities.FerengiRules)
                {
                    rule.ChangeTracker.ChangeTrackingEnabled = true;
                    this.rules.Add(rule);
                }
            }

            this.RulesDataGrid.ItemsSource = this.rules;
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            using (DockOfTheBayEntities entities = new DockOfTheBayEntities())
            {
                // Deletes
                foreach (var rule in this.deletedRules)
                {
                    try
                    {
                        entities.FerengiRules.Attach(rule);
                        entities.FerengiRules.DeleteObject(rule);
                        entities.SaveChanges();
                    }
                    catch (OptimisticConcurrencyException)
                    {
                        switch (this.conflictResolution)
                        {
                            case ConflictResolution.Default:
                                throw;
                            case ConflictResolution.ServerWins:
                                entities.Refresh(RefreshMode.StoreWins, rule);
                                entities.SaveChanges();
                                break;
                            case ConflictResolution.ClientWins:
                                entities.Refresh(RefreshMode.ClientWins, rule);
                                entities.SaveChanges();
                                break;
                            default:
                                break;
                        }
                    }
                }

                // Updates and Inserts
                foreach (var rule in this.rules)
                {
                    try
                    {
                        switch (rule.ChangeTracker.State)
                        {
                            case ObjectState.Added:
                                entities.FerengiRules.AddObject(rule);
                                entities.SaveChanges();
                                break;
                            case ObjectState.Modified:
                                entities.FerengiRules.ApplyChanges(rule);
                                entities.SaveChanges();
                                break;
                            default:
                                break;
                        }
                    }
                    catch (OptimisticConcurrencyException)
                    {
                        switch (this.conflictResolution)
                        {
                            case ConflictResolution.Default:
                                throw;
                            case ConflictResolution.ServerWins:
                                entities.Refresh(RefreshMode.StoreWins, rule);
                                entities.SaveChanges();
                                break;
                            case ConflictResolution.ClientWins:
                                entities.Refresh(RefreshMode.ClientWins, rule);
                                entities.SaveChanges();
                                break;
                            default:
                                break;
                        }
                    }
                }
            }
        }

        private void ResetButton_Click(object sender, RoutedEventArgs e)
        {
            using (DockOfTheBayEntities model = new DockOfTheBayEntities())
            {
                //// Unfortunately the 'GO' in the script is not appreciated:
                //// string script = File.ReadAllText("../../Scripts/Ferengi.sql");
                //// model.ExecuteStoreCommand(script);

                model.ExecuteStoreCommand("truncate table dbo.FerengiRule");

                model.ExecuteStoreCommand(
                  @"INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (1, N'Once you have their money, you never give it back.', N'DS9: ""The Nagus"", ""Heart of Stone""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (3, N'Never spend more for an acquisition than you have to.', N'DS9: ""The Maquis, Part II""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (7, N'Keep your ears open.', N'DS9: ""In the Hands of the Prophets""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (9, N'Opportunity plus instinct equals profit.', N'DS9: ""The Storyteller""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (45, N'Expand, or die.', N'ENT: ""Acquisition""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (48, N'The bigger the smile, the sharper the knife.', N'DS9: ""Rules of Acquisition""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (59, N'Free advice is seldom cheap.', N'DS9: ""Rules of Acquisition""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (74, N'Knowledge equals profit.', N'VOY: ""Inside Man""')
                    INSERT [dbo].[FerengiRule] ([ID], [Text], [Source]) VALUES (217, N'You can''t free a fish from water.', N'DS9: ""Past Tense, Part I""')
                   ");
            }

            this.UndoButton_Click(this, null);
        }

        private void UndoButton_Click(object sender, RoutedEventArgs e)
        {
            this.RefreshRules();
        }

        private void RulesDataGrid_RowDeleted(object sender, ExecutedRoutedEventArgs e)
        {
            if (e.Command == DataGrid.DeleteCommand)
            {
                this.deletedRules.Add((e.Source as DataGrid).SelectedItem as FerengiRule);
            }
        }

        private void Default_Checked(object sender, RoutedEventArgs e)
        {
            this.conflictResolution = ConflictResolution.Default;
        }

        private void Server_Checked(object sender, RoutedEventArgs e)
        {
            this.conflictResolution = ConflictResolution.ServerWins;
        }

        private void Client_Checked(object sender, RoutedEventArgs e)
        {
            this.conflictResolution = ConflictResolution.ClientWins;
        }

        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void Window_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.DragMove();
        }
    }
}